@@ -27,19 +27,17 @@
 # include <stddef.h>		/* size_t */
 # include <unistd.h>		/* ssize_t */
 
-/* GSASL Windows DLL.  Only needed when this file is used in Visual
-   Studio.  Export and import happens automatically in MinGW. */
-#ifndef GSASL_API
-# if defined _MSC_VER && !defined GSASL_STATIC
-#  ifdef GSASL_EXPORTS
+# ifndef GSASL_API
+#  if defined GSASL_BUILDING && defined HAVE_VISIBILITY && HAVE_VISIBILITY
+#   define GSASL_API __attribute__((__visibility__("default")))
+#  elif defined GSASL_BUILDING && defined _MSC_VER && ! defined GSASL_STATIC
 #   define GSASL_API __declspec(dllexport)
-#  else
+#  elif defined _MSC_VER && ! defined GSASL_STATIC
 #   define GSASL_API __declspec(dllimport)
+#  else
+#   define GSASL_API
 #  endif
-# else
-#  define GSASL_API
 # endif
-#endif
 
 # ifdef __cplusplus
 extern "C"
@@ -49,11 +47,55 @@ extern "C"
   /**
    * GSASL_VERSION
    *
-   * String defined via CPP denoting the header file version number.
-   * Used together with stringprep_check_version() to verify header file
-   * and run-time library consistency.
+   * Pre-processor symbol with a string that describe the header file
+   * version number.  Used together with gsasl_check_version() to
+   * verify header file and run-time library consistency.
    */
-# define GSASL_VERSION "1.0"
+# define GSASL_VERSION "1.2"
+
+  /**
+   * GSASL_VERSION_MAJOR
+   *
+   * Pre-processor symbol with a decimal value that describe the major
+   * level of the header file version number.  For example, when the
+   * header version is 1.2.3 this symbol will be 1.
+   *
+   * Since: 1.1
+   */
+# define GSASL_VERSION_MAJOR 1
+
+  /**
+   * GSASL_VERSION_MINOR
+   *
+   * Pre-processor symbol with a decimal value that describe the minor
+   * level of the header file version number.  For example, when the
+   * header version is 1.2.3 this symbol will be 2.
+   *
+   * Since: 1.1
+   */
+# define GSASL_VERSION_MINOR 2
+
+  /**
+   * GSASL_VERSION_PATCH
+   *
+   * Pre-processor symbol with a decimal value that describe the patch
+   * level of the header file version number.  For example, when the
+   * header version is 1.2.3 this symbol will be 3.
+   *
+   * Since: 1.1
+   */
+# define GSASL_VERSION_PATCH 0
+
+  /**
+   * GSASL_VERSION_NUMBER
+   *
+   * Pre-processor symbol with a hexadecimal value describing the
+   * header file version number.  For example, when the header version
+   * is 1.2.3 this symbol will have the value 0x010203.
+   *
+   * Since: 1.1
+   */
+# define GSASL_VERSION_NUMBER 0x010200
 
   /* RFC 2222: SASL mechanisms are named by strings, from 1 to 20
    * characters in length, consisting of upper-case letters, digits,
@@ -154,6 +196,8 @@ extern "C"
     GSASL_PIN = 10,
     GSASL_REALM = 11,
     GSASL_DIGEST_MD5_HASHED_PASSWORD = 12,
+    GSASL_QOPS = 13,
+    GSASL_QOP = 14,
     /* Server validation callback properties. */
     GSASL_VALIDATE_SIMPLE = 500,
     GSASL_VALIDATE_EXTERNAL = 501,
